@@ -392,6 +392,14 @@ function _fix_cxx_compiler_cmake(package, envs)
     end
 end
 
+-- @see https://github.com/ziglang/zig/issues/22213
+function _fix_zigcc_linker_cmake(package, envs)
+    if package:has_tool("cc", "zig_cc") then
+        envs.CMAKE_C_LINKER_DEPFILE_SUPPORTED = "FALSE"
+        envs.CMAKE_CXX_LINKER_DEPFILE_SUPPORTED = "FALSE"
+    end
+end
+
 -- insert configs from envs
 function _insert_configs_from_envs(configs, envs, opt)
     opt = opt or {}
@@ -418,6 +426,11 @@ function _get_configs_for_generic(package, configs, opt)
     if package:is_library() then
         envs.BUILD_SHARED_LIBS = package:config("shared") and "ON" or "OFF"
     end
+    -- https://cmake.org/cmake/help/latest/variable/CMAKE_LINKER_TYPE.html
+    if package:has_tool("ld", "link") then
+        envs.CMAKE_LINKER_TYPE = "MSVC"
+    end
+    _fix_zigcc_linker_cmake(package, envs)
     _insert_configs_from_envs(configs, envs, opt)
 end
 
@@ -873,10 +886,10 @@ function _get_envs_for_flags(package, configs, opt)
         -- use clang-cl or clang, and we need pass --target=xxx flags
         if package:has_tool("cc", "clang", "clang_cl") then
             -- @see https://github.com/xmake-io/xmake-repo/issues/7662
-            platform_envs.CMAKE_C_FLAGS = _get_cflags(package, {cross = true, packagedeps = opt.packagedeps})
+            platform_envs.CMAKE_C_FLAGS = _get_cflags(package, table.join({cross = true}, opt))
         end
         if package:has_tool("cxx", "clang", "clang_cl") then
-            platform_envs.CMAKE_CXX_FLAGS = _get_cxxflags(package, {cross = true, packagedeps = opt.packagedeps})
+            platform_envs.CMAKE_CXX_FLAGS = _get_cxxflags(package, table.join({cross = true}, opt))
         end
     elseif package:is_plat("wasm") then
         -- pass toolchain flags cross-compilation
